# DATA-DRIVEN KERNEL DESIGNS FOR OPTIMIZED GREEDY SCHEMES: A MACHINE LEARNING PERSPECTIVE by
# TIZIAN WENZEL, FRANCESCO MARCHETTI, EMMA PERRACCHIONE
# Code related to the numerical experiment within Section 4.1. "Function approximation
# on the unit cube" to produce the plots within Figure 2.


from utils_code.main_function import run_everything, run_cross_validation
from utils_code.hyperparameters import dic_hyperparams

import numpy as np
from matplotlib import pyplot as plt
from scipy import io
import os
import matplotlib


np.random.seed(1)


## Some settings
# name_dataset = 'example_5d_faster_conv'
name_dataset = 'example_6d_kink'
# name_dataset = 'example_7d_semiactive'


## Load the hyperparameters
hyperparameter = dic_hyperparams[name_dataset]


## Run everything
A_start, A_optimized, model, model_vkoga1, model_vkoga2, data, \
    array_concatenate, _, list_timings_2L = run_everything(
    name_dataset,
    hyperparameter.maxIter_vkoga, hyperparameter.N_points,
    hyperparameter.noise_level, hyperparameter.reg_para_optim, hyperparameter.reg_para_vkoga,
    hyperparameter.learning_rate, hyperparameter.n_epochs,
    hyperparameter.batch_size, hyperparameter.n_folds,
    flag_initialize_diagonal=hyperparameter.flag_initialize_diagonal,
    flag_symmetric_A=hyperparameter.flag_symmetric,
    flag_gaussian=hyperparameter.flag_gaussian)


## Run cross-validation
array_eps, array_cv_f, array_cv_f_val, _, list_timings_1L = run_cross_validation(
    name_dataset, # required only for hyperparameters
    data, hyperparameter.maxIter_vkoga,
    flag_gaussian=hyperparameter.flag_gaussian, n_cross_val=hyperparameter.n_cross_val)



## Store in matlab for beautiful tikzfigure plots
path_for_results = os.path.abspath(os.path.join(os.path.dirname(__file__), 'results_5reruns/')) + '/'

os.makedirs(path_for_results, exist_ok=True)
io.savemat(path_for_results + name_dataset + '.mat',
           dict(array_concatenate=array_concatenate,
                list_objectives=model.list_obj,
                train_hist_deep_f=model_vkoga1.train_hist['f'],
                train_hist_deep_f_val=model_vkoga1.train_hist['f val'],
                train_hist_std_f=model_vkoga2.train_hist['f'],
                train_hist_std_f_val=model_vkoga2.train_hist['f val'],
                array_eps=array_eps,
                array_cv_f=array_cv_f,
                array_cv_f_val=array_cv_f_val,
                list_timings_2L = list_timings_2L,
                list_timings_1L = list_timings_1L))

# in Matlab:
# results = load('gong.mat')


## Some plots
matplotlib.use('TKAgg')

R = np.linspace(0, 1, int(1.5*array_cv_f.shape[1]))
array_color=plt.cm.hsv(R)

# Visualization of the decay of the L-infty error
plt.figure(1)
plt.clf()
plt.plot(model_vkoga1.train_hist['f'], 'k')
# plt.plot(model_vkoga1.train_hist['f val'], 'k--')
for idx_para, para_ep in enumerate(array_eps):
    plt.plot(array_cv_f[:, idx_para], color=array_color[idx_para, :])
    # plt.plot(array_cv_f[:, idx_para], '--', color=array_color[idx_para, :])
plt.title('training error')
plt.xscale('log')
plt.yscale('log')
plt.legend(['2layered'] + [para_ep for para_ep in np.round(array_eps, 3)])
plt.draw()


# Visualization of the evolution of some matrix entries
n_epochs = array_concatenate.shape[-1]

plt.figure(2)
plt.clf()
list_legend = []
for k in range(array_concatenate.shape[0]):
    plt.plot(np.arange(1, n_epochs + 1), array_concatenate[k, k, :])
    list_legend.append('A[{},{}]'.format(k, k))
plt.plot(np.arange(1, n_epochs + 1), array_concatenate[0, 1, :], '--')
plt.plot(np.arange(1, n_epochs + 1), array_concatenate[1, 0, :], '--')
plt.legend(list_legend + ['A[0,1]', 'A[1,0]'])
plt.title('matrix entries')
plt.xscale('log')
plt.xlabel('iterations')
plt.show()



