# Code copied from P07_ML_MORE/Utilities/cv_rippa_ext.py and slightly modified.

import torch


def compute_cv_loss_via_rippa_ext(kernel, X, y, n_folds, reg_for_matrix_inversion):
    """
    Implementation with the need to provide a kernel and points
    """

    # Some precomputations
    kernel_matrix = kernel(X, X) + reg_for_matrix_inversion * torch.eye(X.shape[0])
    inv_kernel_matrix = torch.inverse(kernel_matrix)
    coeffs = torch.linalg.solve(kernel_matrix, y) #[0]

    # Some initializations and preparations: It is required that n_folds divides y.shape[0] without remainder
    array_error = torch.zeros(y.shape[0], 1)
    n_per_fold = int(y.shape[0] / n_folds)
    indices = torch.arange(0, y.shape[0]).view(n_per_fold, n_folds)

    # Standard Rippa's scheme
    if n_folds == y.shape[0]:
        array_error = coeffs / torch.diag(inv_kernel_matrix)

    # Extended Rippa's scheme
    else:
        for j in range(n_folds):
            # inv_kernel_matrix_loc = inv_kernel_matrix[indices[:, j], indices[:, j]]
            inv_kernel_matrix_loc1 = inv_kernel_matrix[indices[:, j], :]
            inv_kernel_matrix_loc = inv_kernel_matrix_loc1[:, indices[:, j]]

            array_error[j * n_per_fold: (j+1) * n_per_fold, 0] = \
                (torch.solve(coeffs[indices[:, j]], inv_kernel_matrix_loc)[0]).view(-1)

    cv_error_sq = torch.sum(array_error ** 2) / array_error.shape[0]

    return cv_error_sq, array_error


def compute_cv_loss_via_rippa_ext_2(kernel_matrix, y, n_folds, reg_for_matrix_inversion):
    """
    Implementation without the need to provide a kernel and points: Simply provide the kernel matrix
    """

    # Some precomputations
    kernel_matrix_reg = kernel_matrix + reg_for_matrix_inversion * torch.eye(kernel_matrix.shape[0])
    inv_kernel_matrix = torch.inverse(kernel_matrix_reg)
    coeffs = torch.linalg.solve(kernel_matrix_reg, y) #[0]

    # Some initializations and preparations: It is required that n_folds divides y.shape[0] without remainder
    array_error = torch.zeros(y.shape[0], 1)
    n_per_fold = int(y.shape[0] / n_folds)
    indices = torch.arange(0, y.shape[0]).view(n_per_fold, n_folds)

    # Standard Rippa's scheme
    if n_folds == y.shape[0]:
        array_error = coeffs / torch.diag(inv_kernel_matrix).view(-1,1)

    # Extended Rippa's scheme
    else:
        for j in range(n_folds):
            # inv_kernel_matrix_loc = inv_kernel_matrix[indices[:, j], indices[:, j]]
            inv_kernel_matrix_loc1 = inv_kernel_matrix[indices[:, j], :]
            inv_kernel_matrix_loc = inv_kernel_matrix_loc1[:, indices[:, j]]

            # array_error[j * n_per_fold: (j+1) * n_per_fold, 0] = \
            #     (torch.solve(coeffs[indices[:, j]], inv_kernel_matrix_loc)[0]).view(-1)
            array_error[j * n_per_fold: (j+1) * n_per_fold, 0] = \
                (torch.linalg.solve(inv_kernel_matrix_loc, coeffs[indices[:, j]])).view(-1)

        # list_for_block = []
        # for j in range(n_folds):
        #     inv_kernel_matrix_loc1 = inv_kernel_matrix[j*n_per_fold : (j+1)*n_per_fold, :]
        #     inv_kernel_matrix_loc = inv_kernel_matrix_loc1[:, j*n_per_fold : (j+1)*n_per_fold]
        #     list_for_block.append(inv_kernel_matrix_loc)
        #
        # from scipy.linalg import block_diag
        # block_matrix = block_diag(list_for_block) # blockmatrix, each block is of size n_per_fold x n_per_fold!
        # array_error2 = torch.linalg.solve(block_matrix, coeffs)

    cv_error_sq = torch.sum(array_error ** 2) / array_error.numel()

    return cv_error_sq, array_error
