import numpy as np
import os

class Dataset():
    """
    Implementation of several toy and real world data sets
    """

    def __init__(self, N_points=10000):

        self.N_points = N_points
        self.X = None
        self.y = None

        self.dic_dataset = {
            'example_5d_faster_conv': (lambda x: np.exp(-4 * np.sum(x[:, :] - .5 * np.ones_like(x[:, :]), axis=1, keepdims=True) ** 2), 5),
            'example_6d_kink': (lambda x: (np.exp(-4 * np.sum((x[:, :] - .5 * np.ones_like(x[:, :])) ** 2, axis=1, keepdims=True)) + 2 * np.abs(x[:, [0]] - .5)), 6),
            'example_7d_semiactive': (lambda x: np.exp(-4 * np.sum((x[:, :] - .5 * np.ones_like(x[:, :])) ** 2, axis=1, keepdims=True))
                                      + np.exp(-9 * np.sum((x[:, :2] - .3 * np.ones_like(x[:, :2])) ** 2, axis=1, keepdims=True)), 7),
        }


    def get_data(self, name_dataset):

        X, y = None, None

        if name_dataset in self.dic_dataset.keys():
            fcn, dim = self.dic_dataset[name_dataset]

            X = np.random.rand(self.N_points, dim)
            y = fcn(X)
        elif name_dataset in ['ct', 'diamonds', 'fried', 'kegg_undir_uci',
                              'methane', 'mlr_knn_rng', 'online_video', 'poker',
                              'protein', 'query_agg_count', 'road_network', 'sarcos', 'sgemm',
                              'stock', 'wecs']:
            X, y = self.example_holzmuller(name_dataset)
        else:
            pass

        assert X is not None, 'Bug in get_data!'
        assert y is not None, 'Bug in get_data!'

        return X, y


    def example_holzmuller(self, ds_name):
        """Data set from David Holzmüller paper of batch active learning."""

        path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'data/data/')) + '/'

        X = np.load(path + ds_name + '/X.npy')
        y = np.load(path + ds_name + '/y.npy')

        # Standardization of input according to David H (reduce effect of outliers)
        X = (X - X.mean(axis=0, keepdims=True)) / (X.std(axis=0, keepdims=True) + 1e-30)
        X = 5 * np.tanh(0.2 * X)

        y = (y - y.mean(axis=0, keepdims=True)) / (y.std(axis=0, keepdims=True) + 1e-30)

        return X, y

