from pathlib import Path
from typing import *

from utils_data import custom_paths
from utils_data import utils


class DataInfo:
    """
    Represents information about a data set.
    """
    def __init__(self, ds_name: str, n_tvp: int, n_test: int, n_features: int,
                 train_test_split: Optional[int]):
        """
        :param ds_name: Name of the data set.
        :param n_tvp: Number of train+val+pool samples.
        :param n_test: Number of test samples.
        :param n_features: Number of input features of the data set.
        :param train_test_split: Set to None if the data set does not have a fixed (train+val+pool)-test split.
        If this is an int, it is interpreted such that the train+val+pool set are the first train_test_split samples
        and the test set are the remaining samples.
        """
        # tvp = train+val+pool
        self.ds_name = ds_name
        self.n_features = n_features
        self.n_tvp = n_tvp
        self.n_test = n_test
        self.n_samples = n_tvp + n_test
        self.train_test_split = train_test_split

    def save(self):
        """
        Saves this object to the path of the corresponding data set.
        """
        utils.serialize(Path(custom_paths.get_data_path()) / 'data' / self.ds_name / 'data_info.pkl', self)